@@ -78,13 +78,14 @@ lid_switch_keyboard_event(uint64_t time,
 		return;
 
 	if (dispatch->reliability == RELIABILITY_WRITE_OPEN) {
+		__attribute__((unused)) int rc;
 		int fd = libevdev_get_fd(dispatch->device->evdev);
 		struct input_event ev[2] = {
 			{{ 0, 0 }, EV_SW, SW_LID, 0 },
 			{{ 0, 0 }, EV_SYN, SYN_REPORT, 0 },
 		};
 
-		(void)write(fd, ev, sizeof(ev));
+		rc = write(fd, ev, sizeof(ev));
 		/* In case write() fails, we sync the lid state manually
 		 * regardless. */
 	}
@@ -104,6 +105,8 @@ lid_switch_toggle_keyboard_listener(struct lid_switch_dispatch *dispatch,
 	if (!dispatch->keyboard.keyboard)
 		return;
 
+	libinput_device_remove_event_listener(&dispatch->keyboard.listener);
+
 	if (is_closed) {
 		libinput_device_add_event_listener(
 					&dispatch->keyboard.keyboard->base,
@@ -111,10 +114,7 @@ lid_switch_toggle_keyboard_listener(struct lid_switch_dispatch *dispatch,
 					lid_switch_keyboard_event,
 					dispatch);
 	} else {
-		libinput_device_remove_event_listener(
-					&dispatch->keyboard.listener);
-		libinput_device_init_event_listener(
-					&dispatch->keyboard.listener);
+		libinput_device_init_event_listener(&dispatch->keyboard.listener);
 	}
 }
 
@@ -130,11 +130,12 @@ lid_switch_process_switch(struct lid_switch_dispatch *dispatch,
 	case SW_LID:
 		is_closed = !!e->value;
 
-		if (dispatch->lid_is_closed == is_closed)
-			return;
 		lid_switch_toggle_keyboard_listener(dispatch,
 						    is_closed);
 
+		if (dispatch->lid_is_closed == is_closed)
+			return;
+
 		dispatch->lid_is_closed = is_closed;
 
 		lid_switch_notify_toggle(dispatch, device, time);
