@@ -52,6 +52,8 @@ list_insert(struct list *list, struct list *elm)
 {
 	assert((list->next != NULL && list->prev != NULL) ||
 	       !"list->next|prev is NULL, possibly missing list_init()");
+	assert(((elm->next == NULL && elm->prev == NULL) || list_empty(elm)) ||
+	       !"elm->next|prev is not NULL, list node used twice?");
 
 	elm->prev = list;
 	elm->next = list->next;
