@@ -354,27 +354,21 @@ print_device_notify(struct libinput_event *ev)
 static inline void
 usage(void)
 {
-	printf("Usage: %s [--help|--version]\n"
-	       "\n"
-	       "This tool creates a libinput context on the default seat \"seat0\"\n"
-	       "and lists all devices recognized by libinput and the configuration options.\n"
-	       "Where multiple options are possible, the default is prefixed with \"*\".\n"
-	       "\n"
-	       "Options:\n"
-	       "--help ...... show this help\n"
-	       "--version ... show version information\n"
-	       "\n"
-	       "This tool requires access to the /dev/input/eventX nodes.\n",
-	       program_invocation_short_name);
+	printf("Usage: libinput list-devices [--help|--version]\n");
+	printf("\n"
+	       "--help ...... show this help and exit\n"
+	       "--version ... show version information and exit\n"
+	       "\n");
 }
 
 int
 main(int argc, char **argv)
 {
 	struct libinput *li;
-	struct tools_context context;
 	struct libinput_event *ev;
 
+	/* This is kept for backwards-compatibility with the old
+	   libinput-list-devices */
 	if (argc > 1) {
 		if (streq(argv[1], "--help")) {
 			usage();
@@ -388,9 +382,7 @@ main(int argc, char **argv)
 		}
 	}
 
-	tools_init_context(&context);
-
-	li = tools_open_backend(&context);
+	li = tools_open_backend(BACKEND_UDEV, "seat0", false, false);
 	if (!li)
 		return 1;
 
@@ -406,5 +398,5 @@ main(int argc, char **argv)
 
 	libinput_unref(li);
 
-	return 0;
+	return EXIT_SUCCESS;
 }
